#!/bin/ksh
#
# Copyright (C) Tivoli Systems/IBM. <2000, 2001>
# Unpublished Work
# All Rights Reserved
# Licensed Material - Property of Tivoli Systems/IBM.
#
#set -x

####################################################################
####################################################################
#
# Data collection script to extract useful data for diagnosing
# potential PDOS problems.
#
####################################################################
####################################################################


PROGRAM=kossdump.sh
ISLIVE=0
DOFULL=1
PDOSKERN="/opt/pdos/kernel"
ACTIVE_KERNEL="/dev/ksyms"
D=$ACTIVE_KERNEL
U=$ACTIVE_KERNEL
PDOSBIN="/opt/pdos/bin"
PDOSINST="/opt/pdos"


##########################################################
##########################################################
# Functions common to all platforms
##########################################################
##########################################################

major_section() {
    stitle=$1
    print ""
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOO  $stitle"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print "OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
    print ""
}


check_exempt()
{
    #
    # Try to become exempt for speed and to minimize change in PDOS state
    #
    /opt/pdos/bin/pdosctl -s pdosd -q
    PDOSD_UP=$?
    if [ $PDOSD_UP = 0 ] ; then
      print "\nRunning pid is $$"
      /opt/pdos/bin/pdosexempt -i $$
      if [ $? -ne 0 ] ; then
	print "1st exempt attempt did not work, try with an su"
        su - osseal -c /opt/pdos/bin/pdosexempt -i $$
      fi
      /opt/pdos/bin/pdoswhoami -l
      /opt/pdos/bin/pdoswhois -l $$
    fi
}


usage() {
  print "usage: $PROGRAM [-d dump_image] [-k kernel] [-b] [-h]"
  print "  -d  platform specific location of system dump image"
  print "      data is collected from the running system by default"
  print "  -k  location of kernel binary image if applicable"
  print "  -b  collect abbreviated kernel data"
  print "  -h  display this usage information"

  print "\n"
  print "$PROGRAM collects PDOS related data for use in problem determination."

  print "Suggested usage examples:"
  print "\nFor general PDOS data collection on a running system:"
  print "  $PROGRAM -b > /tmp/pdosinfo.out"

  print "\nIf applications appear to be blocked or running very slow:"
  print "  $PROGRAM > /tmp/pdosinfo.out"

  print "\nIn the event of a system failure which produces a system dump:"
  print "  $PROGRAM -d dump_path -k kernel_path > /tmp/pdosinfo.out"

  print "\nCompression of the output file using compress or gzip is"
  print "recommended before sending the information to customer support."

  exit 1
}


#
# Handle optional arguments
#
ProcessArgs () {

  while getopts "hbk:d:" opt; do
    case $opt in
      k)
        U=$OPTARG;;
      d)  
        D=$OPTARG;;
      b)  
        DOFULL=0;;
      h)
	usage;;
      \?)
	usage;;
    esac
  done

  shift $(($OPTIND - 1))

  if [ $# -ne 0 ] ; then
    print "bad arg count $#"
    usage
  fi

#
# Do some general checking
#
  if [ $D = $ACTIVE_KERNEL ] ; then
    ISLIVE=1
  fi

  print "\n\nPoint of collection: `hostname`"
  print "Collection date: `date`"

  if [ -f /usr/xpg4/bin/id ] ; then
    EFFID=`/usr/xpg4/bin/id -u`
  else
    EFFID=`/usr/bin/id -u`
  fi

  if [ $EFFID != 0 ] ; then
    print "Warning: collection of PDOS information should be done as root"
    print -u2 "Warning: collection of PDOS information should be done as root"
  fi
}


get_pdos_info() {
    #
    # Collect general PDOS non kernel, S/W levels, and config info.
    #
    SUM=/usr/bin/sum
    WHAT=/usr/ccs/bin/what
    LS=/usr/bin/ls
    CAT=/usr/bin/cat
    DF="/usr/bin/df -k"
    TAIL="/usr/bin/tail"

    major_section "General PDOS Information"

    print ">>> PDOS Kernel sums:"
    find /opt/pdos/kernel -type f -exec $SUM {} \;
    if [ -f /usr/lib/drivers/kazn.ext ] ; then
        $SUM /usr/lib/drivers/kazn.ext
    fi

    if [ -f /usr/lib/drivers/pse/kazntli ] ; then
        $SUM /usr/lib/drivers/pse/kazntli*
    fi

    print ""
    print ">>> PDOS bin and lib sums:"
    find /opt/pdos/bin -type f -exec $SUM {} \;
    find /opt/pdos/lib -type f -exec $SUM {} \;
    print ""
    print ">>> Source version info"
    find /opt/pdos/bin -type f -exec $WHAT {} \; |fgrep -v SMI
    print ""
    find /opt/pdos/lib -type f -exec $WHAT {} \; | fgrep -v SMI
    print ""
    find /opt/pdos/kernel -type f -exec $WHAT {} \; | fgrep -v SMI
    print ""
    print ""
    print ""
#
#
    print ">>> PDOS configuration info:\n"
    $LS -l /opt/pdos/etc

    print "\n\n>>>osseal.conf contents:"
    $CAT /opt/pdos/etc/osseal.conf
    print "\n\n>>> pdosd.conf contents:"
    $CAT /opt/pdos/etc/pdosd.conf
    print "\n\n>>> lpm.conf contents:"
    $CAT /opt/pdos/etc/lpm.conf
#
#
    print "\n\n>>> /var/pdos dir contents (ls -lisR):"
    $LS -lsiR /var/pdos

    print "\n\n>>> Free space in major PDOS data directories:"
    print "/var/pdos/azn:"
    $DF /var/pdos/azn
    print "/var/pdos/audit:"
    $DF /var/pdos/audit
    print "/var/pdos/cred:"
    $DF /var/pdos/cred
    print "/var/pdos/log:"
    $DF /var/pdos/log

    print "\n\n>>> pdosd log file data":
    for i in /var/pdos/log/pdosd.log*
    do
        print "+++ logfile: $i +++"
	$TAIL -200 $i
    done

    print "\n\n>>> pdosauditd log file data":
    for i in /var/pdos/log/pdosauditd.log*
    do
        print "+++ logfile: $i +++"
	$TAIL -200 $i
    done

    print "\n\n>>> pdoswdd log file data":
    for i in /var/pdos/log/pdoswdd.log*
    do
        print "+++ logfile: $i +++"
	$TAIL -100 $i
    done
}


get_kazn_stats()
{
    major_section "PDOS Kernel trace, stats, and error info"
    if [ $ISLIVE = 1 ] ; then
      print "\n\n>>> PDOS Kernel trace data (kazntrace):"
      $PDOSKERN/kazntrace -l
      print "\n\n\n"
      $PDOSKERN/kazntrace

      print "\n\n>>> PDOS Kernel statistics data:"
      $PDOSKERN/kossinfo
      print ""
      $PDOSKERN/kossinfo -x
#     $PDOSKERN/kossinfo -k
    fi

    print "\n\n>>> kosserrs.log:"
    $PDOSKERN/kosserrs
    if [ -s /var/pdos/log/kosserrs.log.old ] ; then
      print "\n>>> previous kosserrs.log:"
      $PDOSKERN/kosserrs -f /var/pdos/log/kosserrs.log.old
    fi
}


#####################################################
#####################################################
#
# Functions that me be somewhat platform specfic
#
#####################################################
#####################################################


get_system_info()
{
    major_section "Local System Information"
    print "\n>>> uname info:"
    uname -a
    print ""

    if [ $ISLIVE = 1 ] ; then
      print ">>> Current running processes (ps -elf):"
      /usr/bin/ps -elf

      print "\n\n>>> Live vmstat and iostat samples"
      /usr/bin/vmstat 2 5
      /usr/bin/iostat 2 5
    fi

    print "\n\n>>> Networking Information:"
    print "netstat -in ..."
    netstat -in
    print "\nnetstat -s ..."
    netstat -s
    print "\nifconfig..."
    ifconfig -a

    print "\n\n>>> File System Information"
    /usr/sbin/mount
    print ""
    /usr/bin/df -k

    print "\n\n>>> S/W products:"
    showrev -a
    print ""
    sysdef -D $D

    print "\n>>> System (/etc/system) configuration parameters:"
    cat /etc/system | egrep -v  ^[*] | egrep [a-zA-Z]

    print "\n>>> /var/adm/messages data:"
    tail -200 /var/adm/messages

    print "\n>>> Loaded drivers:"
    modinfo
}


get_hw_info()
{
    major_section "H/W Related Information"

    prtconf -P
    print ""
    psrinfo -v
}



##################################################################
##################################################################
##################################################################
#
# Platform specific functions to extract data from running
# kernels or system dumps.
#
##################################################################
##################################################################


get_kernel_data()
{
    major_section " *** COLLECT PDOS kernel related data ***"
    print -u2 "Collecting PDOS kernel related data. This may take a while..."

    if [ $ISLIVE = 1 ] ; then
      print "\n>>> Extract data from running kernel image"
      ADBCMD="/usr/bin/adb -k $D"
    else
      print "\n>>> Extract data from system dump image"
      print "    dump spec: $D"
      print "    kernel spec: $U"
      ADBCMD="/usr/bin/adb -k $U $D"
    fi
    print "\n\n"

    $ADBCMD << EOF
="Initial Dump Information"
="========================"
$<<utsname
srpc_domain/s15t"Domain name"
time/Y15t"Time of crash"
lbolt>a
*time-(*<a%0t100)=Y15t"Time of boot"
*panicstr/n"Panic string:"ts
=n"Message Buffer:"
$<<msgbuf
=n"Stack traceback"
=n
$ C
=n"Full stack traceback with registers:"
<sp$<<stacktrace
=n"VFS info:"
$<<mount
$<<vfslist
=n"Kernel Memory info:"
$<<kmastat
=n"C2 Auditing Subsystem Active?:"
audit_active/X
=n"ooooooooooooo PDOS internal data oooooooooooooo"
=n"kazn stats:"
kaznStats/30D
=n"kpcfile stats:"
kpcFileStats/40D
=n"kazn rc stats:"
kazn_rcfStats/12D
kazn_rcIdStats/8D
=n"kmsg devdata:"
kmsg_devData/60D
=n"pal memStats:"
pal_memStats/10D
pal_sysMemMB/1D
=n"kenv proc stats:"
kenv_procStats/4D
=n"kaud info:"
kaud_audit_level/1X
kaud_warning_mode/1X
kaud_intTraceFlags/1X
=n"Misc info:"
kutil_kossActive/1X
kutil_kossWasActive/1X
kenv_procRegisteredProcs/1D
kutil_privUid/2D
kutil_kossDebugFlag/1D
kazn_remappedTimeouts/1D
=n"TLI info:"
koss_tliStats/28D
EOF

if [ $DOFULL = 1 ] ; then
    $ADBCMD << EOF
="Get all thread stacks"
="========================"
=n"All running threads:"
$<<threadlist
<sp$<<stacktrace
EOF
fi

    OSLEVEL=`uname -r`
    if [[ $OSLEVEL == "5.7" || $OSLEVEL == "5.8" ]] ; then
      # Check the arch support
      isainfo | grep sparcv9 > /dev/null
      if [ $? -eq 0 ] ; then
  	ARCH="sparcv9/"
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=64_27
	else
	    BITS=64_28
	fi
      else
	if [[ $OSLEVEL == "5.7" ]] ; then
	    BITS=32_27
	else
	    BITS=32_28
	fi
      fi
    else
      BITS=32_26
    fi
    print "\n>>> bit and os: $BITS"
    print "\n>>> End of kernel data extraction"
}


####################################################################
####################################################################
####################################################################
####################################################################
# Main Starting point
####################################################################

 #####  #######    #    ######  #######
#     #    #      # #   #     #    #
#          #     #   #  #     #    #
 #####     #    #     # ######     #
      #    #    ####### #   #      #
#     #    #    #     # #    #     #
 #####     #    #     # #     #    #

#     #    #      ###   #     #
##   ##   # #      #    ##    #
# # # #  #   #     #    # #   #
#  #  # #     #    #    #  #  #
#     # #######    #    #   # #
#     # #     #    #    #    ##
#     # #     #   ###   #     #

####################################################################
####################################################################
####################################################################

PROGRAM=`basename $0`
ProcessArgs $@

if [ $U != 0 ] ; then
    if [ $D = $ACTIVE_KERNEL -a $U != $ACTIVE_KERNEL ] ; then
      print "A dump image spec (-d) must be specified with -k"
      usage
    fi
fi
  
if [ $D != $ACTIVE_KERNEL -a $U = $ACTIVE_KERNEL ] ; then
    print "a kernel spec (-k) must be specified with -d"
    usage
fi

print -u2 ".... Collecting PDOS information ...."
major_section "PDOS data collection utility"

check_exempt
get_kazn_stats
print -u2 "."

if [ $U != 0 ] ; then
    get_kernel_data
    print -u2 "."
fi

get_system_info
print -u2 "."
get_hw_info
print -u2 "."
get_pdos_info

print -u2 "Done..."
print "\n\n>>> End of Collected Data"

exit 0
